import sys
from PyQt4 import QtGui
from Orange.widgets.widget import OWWidget
from Orange.widgets import widget, gui, settings
from orangecontrib.datafusion.models import Relation
from orangecontrib.datafusion import movielens

from skfusion import fusion

class OWMovieRatings(OWWidget):
    name = "Movie Ratings"
    priority = 70000
    icon = "icons/MovieRatings.svg"
    want_main_area = False
    description = "Get a matrix of user ratings for movies."
    outputs = [("Ratings", Relation, widget.Default)]

    percent = settings.Setting(10)  # 10: default is 10% of data
    start = settings.Setting(2005)  # from year 2005 (default)
    end = settings.Setting(2007)    # to year 2007 (default)
    method = settings.Setting(0)    # 0: percentage, 1: years

    def __init__(self):
        super().__init__()

        box = gui.widgetBox(self.controlArea, "Select Subset")
        methodbox = gui.radioButtons(
            box, self, "method", callback=self._on_method_changed)

        gui.appendRadioButton(methodbox, "Percentage:")
        percent = gui.hSlider(
            gui.indentedBox(methodbox), self, "percent",
            minValue=1, maxValue=100, step=1, ticks=10, labelFormat="%d %%")

        gui.appendRadioButton(methodbox, "Time period:")
        ibox = gui.indentedBox(methodbox)
        start = gui.spin(
            ibox, self, "start", 1907, 2015, 1, label="Starting year: ")
        end = gui.spin(
            ibox, self, "end", 1907, 2015, 1, label="Ending year: ")

        self.method_params = [percent, start, end]

        gui.button(self.controlArea, self, "&Apply",
                   callback=self.send_output, default=True)

        self.setSizePolicy(QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,
                           QtGui.QSizePolicy.Fixed))

        self.setMinimumWidth(250)
        self.setMaximumWidth(250)
        self._on_method_changed()
        self.send_output()

    def _on_method_changed(self):
        enabled = [[True, False, False], [False, True, True]]
        mask = enabled[self.method]
        for param, enabled in zip(self.method_params, mask):
            param.setEnabled(enabled)

    def send_output(self):
        if self.method == 0:
            matrix, movies, users = movielens.movie_user_matrix(percentage=self.percent)
        else:
            try:
                matrix, movies, users = movielens.movie_user_matrix(start_year=self.start, end_year=self.end)
            except ValueError:
                self.error(0, "Invalid starting years")
                self.send("Ratings", None)

        def scale(X):
            return (X - X.min()) / (X.max() - X.min())

        relation = fusion.Relation(matrix.T, name='rate',
                                   row_type=movielens.ObjectType.Users, row_names=users,
                                   col_type=movielens.ObjectType.Movies, col_names=movies,
                                   preprocessor=scale)
        self.send("Ratings", Relation(relation))

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    ow = OWMovieRatings()
    ow.show()
    app.exec_()



from io import BytesIO
from collections import defaultdict
from os import path

from PyQt4 import QtCore, QtGui, QtWebKit

from Orange.widgets import widget, gui, settings
from skfusion import fusion
from orangecontrib.datafusion.models import Relation, FittedFusionGraph


JS_GRAPH = open(path.join(path.dirname(__file__), 'graph_script.js')).read()

import re

DECOMPOSITION_ALGO = [
    ('Matrix tri-factorization', fusion.Dfmf),
    ('Matrix tri-completion', fusion.Dfmc),
]
INITIALIZATION_ALGO = [
    'Random',
    'Random C',
    'Random Vcol'
]


class Output:
    RELATION = 'Relation'
    FUSION_GRAPH = 'Fusion Graph'
    FUSER = 'Fitted Fusion Graph'


def rel_shape(relation):
    return '{}×{}'.format(*relation.shape)


def rel_cols(relation):
    return [relation.row_type.name,
            relation.name or '→',
            relation.col_type.name]


def relation_str(relation):
    return '[{}] {}'.format(rel_shape(relation.data), ' '.join(rel_cols(relation)))


def _get_selected_nodes(element_id, graph):
    """ Return ObjectTypes from FusionGraph `graph` that correspond to
        selected `element_id` in the webview.
    """
    selected_is_edge = element_id.startswith('edge ')
    assert element_id.startswith('edge ') or element_id.startswith('node ')
    # Assumes SVG element's id attributes specify nodes `-delimited
    node_names = re.findall('`([^`]+)`', element_id)
    nodes = [graph.get_object_type(name) for name in node_names]
    assert len(nodes) == 2 if selected_is_edge else len(nodes) == 1
    return nodes


class WebviewWidget(QtWebKit.QWebView):
    def __init__(self, parent):
        super().__init__(parent)
        parent.layout().addWidget(self)
        settings = self.settings()
        if __debug__:  # TODO
            settings.setAttribute(settings.DeveloperExtrasEnabled, True)
        else:
            self.setContextMenuPolicy(QtCore.Qt.NoContextMenu)

    def sizeHint(self):
        return QtCore.QSize(500, 500)

    def evalJS(self, javascript):
        self.page().mainFrame().evaluateJavaScript(javascript)

    def repaint(self, graph, parent):
        stream = BytesIO()
        graph.draw_graphviz(stream, 'svg')
        stream.seek(0)
        stream = QtCore.QByteArray(stream.read())
        self.setContent(stream, 'image/svg+xml')
        webframe = self.page().mainFrame()
        webframe.addToJavaScriptWindowObject('pybridge', parent)
        webframe.evaluateJavaScript(JS_GRAPH)


class SimpleTableWidget(QtGui.QTableWidget):
    """ A wrapper around QTableWidget """
    def __init__(self, parent=None, callback=None):
        """`callback` is a function that accepts first selected row item"""
        super().__init__(parent)
        self.callback = callback
        self.horizontalHeader().setStretchLastSection(True)
        self.horizontalHeader().setVisible(False)
        self.verticalHeader().setVisible(False)
        self.setHorizontalScrollMode(self.ScrollPerPixel)
        self.setVerticalScrollMode(self.ScrollPerPixel)
        self.setSelectionMode(self.SingleSelection)
        self.setSelectionBehavior(self.SelectRows)
        self.setEditTriggers(self.NoEditTriggers)
        self.setAlternatingRowColors(True)
        self.setShowGrid(False)
        self.currentItemChanged.connect(self._on_currentItemChanged)
        if parent: parent.layout().addWidget(self)

    def add(self, items, bold=()):
        """Appends iterable of `items` as the next row.

        `bold` is a list of columns that are to be set in bold face.
        """
        row = self.rowCount()
        self.insertRow(row)
        self.setColumnCount(max(len(items), self.columnCount()))
        for col, data in enumerate(items):
            try: name, data = data
            except ValueError: name = str(data)
            item = QtGui.QTableWidgetItem(name)
            item.setData(QtCore.Qt.UserRole, data)
            if col in bold:
                font = item.font()
                font.setBold(True)
                item.setFont(font)
            self.setItem(row, col, item)
        self.resizeColumnsToContents()
        self.resizeRowsToContents()

    def clear(self):
        super().clear()
        self.setRowCount(0)
        self.setColumnCount(0)

    def select_first(self):
        if self.rowCount() > 0:
            self.selectRow(0)

    def _on_currentItemChanged(self, current, previous):
        if self.callback and current:
            item = self.item(current.row(), 0)
            return self.callback(item)


LIMIT_RANK_THRESHOLD = 1000  # If so many objects or more, limit maximum rank


class OWFusionGraph(widget.OWWidget):
    name = "Fusion Graph"
    priority = 10000
    icon = "icons/FusionGraph.svg"
    inputs = [("Relation", Relation, "on_relation_change", widget.Multiple)]
    outputs = [
        (Output.RELATION, Relation),
        (Output.FUSER, FittedFusionGraph, widget.Default),
        (Output.FUSION_GRAPH, fusion.FusionGraph),
    ]

    # Signal emitted when a node in the SVG is selected, carrying its name
    graph_element_selected = QtCore.pyqtSignal(str)

    pref_algo_name = settings.Setting('')
    pref_algorithm = settings.Setting(0)
    pref_initialization = settings.Setting(0)
    pref_n_iterations = settings.Setting(10)
    pref_rank = settings.Setting(10)
    autorun = settings.Setting(False)

    def __init__(self):
        super().__init__()
        self.n_object_types = 0
        self.n_relations = 0
        self.relations = {}  # id-->relation map
        self.graph_element_selected.connect(self.on_graph_element_selected)
        self.graph = fusion.FusionGraph()
        self.webview = WebviewWidget(self.mainArea)
        self._create_layout()

    @QtCore.pyqtSlot(str)
    def on_graph_element_selected(self, element_id):
        """Handle self.graph_element_selected signal, and highlight also:
           * if edge was selected, the two related nodes,
           * if node was selected, all its edges.
           Additionally, update the info box.
        """
        if not element_id:
            return self._populate_table()
        selected_is_edge = element_id.startswith('edge ')
        nodes = _get_selected_nodes(element_id, self.graph)
        # CSS selector query for selection-relevant nodes
        selector = ','.join('[id^="node "][id*="`%s`"]' % n.name for n in nodes)
        # If a node was selected, include the edges that connect to it
        if not selected_is_edge:
            selector += ',[id^="edge "][id*="`%s`"]' % nodes[0].name
        # Highlight these additional elements
        self.webview.evalJS("highlight('%s');" % selector)
        # Update the control table table
        if selected_is_edge:
            relations = self.graph.get_relations(*nodes)
        else:
            relations = (set(i for i in self.graph.in_relations(nodes[0])) |
                         set(i for i in self.graph.out_relations(nodes[0])))
        self._populate_table(relations)

    def _create_layout(self):
        info = gui.widgetBox(self.controlArea, 'Info')
        gui.label(info, self, '%(n_object_types)d object types')
        gui.label(info, self, '%(n_relations)d relations')
        # Table view of relation details
        info = gui.widgetBox(self.controlArea, 'Relations')

        def send_relation(item):
            data = item.data(QtCore.Qt.UserRole)
            self.send(Output.RELATION, Relation(data))

        self.table = SimpleTableWidget(info, callback=send_relation)
        self.controlArea.layout().addStretch(1)
        gui.lineEdit(self.controlArea,
                     self, 'pref_algo_name', 'Fuser name',
                     callback=self.checkcommit, enterPlaceholder=True)
        gui.radioButtons(self.controlArea,
                         self, 'pref_algorithm', [i[0] for i in DECOMPOSITION_ALGO],
                         box='Decomposition algorithm',
                         callback=self.checkcommit)
        gui.radioButtons(self.controlArea,
                         self, 'pref_initialization', INITIALIZATION_ALGO,
                         box='Initialization algorithm',
                         callback=self.checkcommit)
        gui.hSlider(self.controlArea, self, 'pref_n_iterations',
                    'Maximum number of iterations',
                    minValue=10, maxValue=500, createLabel=True,
                    callback=self.checkcommit)
        self.slider_rank = gui.hSlider(self.controlArea, self, 'pref_rank',
                                       'Factorization rank',
                                       minValue=1, maxValue=100, createLabel=True,
                                       labelFormat=" %d%%",
                                       callback=self.checkcommit)
        gui.auto_commit(self.controlArea, self, "autorun", "Run",
                        checkbox_label="Run after any change  ")

    def checkcommit(self):
        return self.commit()

    def commit(self):
        Algo = DECOMPOSITION_ALGO[self.pref_algorithm][1]
        init_type = INITIALIZATION_ALGO[self.pref_initialization].lower().replace(' ', '_')
        # Update rank on object-types
        maxrank = defaultdict(int)
        for rel in self.graph.relations:
            rows, cols = rel.data.shape
            row_type, col_type = rel.row_type, rel.col_type
            if rows > maxrank[row_type]:
                maxrank[row_type] = row_type.rank = max(5, int(rows * (self.pref_rank / 100)))
            if cols > maxrank[col_type]:
                maxrank[col_type] = col_type.rank = max(5, int(cols * (self.pref_rank / 100)))
        # Run the algo ...
        self.fuser = Algo(init_type=init_type,
                          max_iter=self.pref_n_iterations, random_state=0).fuse(self.graph)
        self.fuser.name = self.pref_algo_name
        self.send(Output.FUSER, FittedFusionGraph(self.fuser))

    def _populate_table(self, relations=None):
        self.table.clear()
        for i in relations or self.graph.relations:
            self.table.add([(rel_shape(i.data), i)] + rel_cols(i), bold=(1, 3))
        self.table.select_first()

    def on_relation_change(self, relation, id):
        def _on_remove_relation(id):
            try: relation = self.relations.pop(id)
            except KeyError: return
            self.graph.remove_relation(relation)

        def _on_add_relation(relation, id):
            _on_remove_relation(id)
            self.relations[id] = relation
            self.graph.add_relation(relation)

        if relation:
            _on_add_relation(relation.relation, id)
        else:
            _on_remove_relation(id)
        self._populate_table()
        self.slider_rank.setMaximum(30
                                    if any(max(rel.data.shape) > LIMIT_RANK_THRESHOLD
                                           for rel in self.graph.relations)
                                    else
                                    100)
        self.webview.repaint(self.graph, self)
        self.send(Output.FUSION_GRAPH, self.graph)
        # this ensures gui.label-s get updated
        self.n_object_types = self.graph.n_object_types
        self.n_relations = self.graph.n_relations

    # called when all signals are received, so the graph is updated only once
    def handleNewSignals(self):
        self.unconditional_commit()

def main():
    # example from https://github.com/marinkaz/scikit-fusion
    import numpy as np
    R12 = np.random.rand(50, 100)
    R22 = np.random.rand(100, 100)
    R13 = np.random.rand(50, 40)
    R31 = np.random.rand(40, 50)
    R23 = np.random.rand(100, 40)
    R23 = np.random.rand(100, 40)
    R24 = np.random.rand(100, 400)
    R34 = np.random.rand(40, 400)
    t1 = fusion.ObjectType('Users', 10)
    t2 = fusion.ObjectType('Actors', 20)
    t3 = fusion.ObjectType('Movies', 30)
    t4 = fusion.ObjectType('Genres', 40)
    relations = [fusion.Relation(R12, t1, t2, name='like'),
                 fusion.Relation(R13, t1, t3, name='rated'),
                 fusion.Relation(R23, t2, t3, name='play in'),
                 fusion.Relation(R31, t3, t1),
                 fusion.Relation(R24, t2, t4, name='prefer'),
                 fusion.Relation(R34, t3, t4, name='belong to'),
                 fusion.Relation(R22, t2, t2, name='married to')]

    app = QtGui.QApplication(['asdf'])
    w = OWFusionGraph()
    w.show()

    def _add_next_relation(event,
                           id=iter(range(len(relations))),
                           relation=iter(map(Relation, relations))):
        try: w.on_relation_change(next(relation), next(id))
        except StopIteration:
            w.killTimer(w.timer_id)
            w.on_relation_change(None, 4)  # Remove relation #4
    w.timerEvent = _add_next_relation
    w.timer_id = w.startTimer(500)
    app.exec()


if __name__ == "__main__":
    main()



from os import path

from PyQt4 import QtCore, QtGui

from Orange.widgets import widget, gui, settings
from skfusion import fusion
from orangecontrib.datafusion.widgets.owfusiongraph import \
    WebviewWidget, rel_shape, rel_cols, _get_selected_nodes, SimpleTableWidget
from orangecontrib.datafusion.models import Relation, FittedFusionGraph

JS_FACTORS = open(path.join(path.dirname(__file__), 'factors_script.js')).read()


def is_constraint(relation):
    """Skip constraint (Theta) relations"""
    return relation.row_type == relation.col_type


class Output:
    RELATION = 'Relation'


class OWLatentFactors(widget.OWWidget):
    name = "Latent Factors"
    priority = 20000
    icon = "icons/LatentFactors.svg"
    inputs = [("Fitted fusion graph", FittedFusionGraph, "on_fuser_change")]
    outputs = [(Output.RELATION, Relation)]

    # Signal emitted when a node in the SVG is selected, carrying its id
    graph_element_selected = QtCore.pyqtSignal(str)
    # Signal to communicate the sizes of matrices on a node / along an edge
    graph_element_get_size = QtCore.pyqtSignal(str)

    autorun = settings.Setting(True)

    completions = settings.Setting([])
    selected_completions = settings.Setting([])
    factors = settings.Setting([])
    selected_factors = settings.Setting([])
    backbones = settings.Setting([])
    selected_backbones = settings.Setting([])

    def __init__(self):
        super().__init__()
        self.n_relations = 0
        self.n_object_types = 0
        self.graph_element_selected.connect(self._on_graph_element_selected)
        self.graph_element_get_size.connect(self.on_graph_element_get_size)
        self.webview = WebviewWidget(self.mainArea)
        self._create_layout()

    @QtCore.pyqtSlot(str)
    def on_graph_element_get_size(self, element_id):
        """ Return the list of matrix.shape[0] for the selected element(s)
            (node(=factor) or edge(=backbone)).
        """
        selected_is_edge = element_id.startswith('edge ')
        nodes = _get_selected_nodes(element_id, self.fuser)
        from math import log2

        def _norm(s):
            return min(max(1.3**log2(s), 8), 20)

        if selected_is_edge:
            rels = self.fuser.get_relations(nodes[0], nodes[1])
            sizes = [_norm(self.fuser.backbone(rel).shape[0])
                     for rel in rels
                     if not is_constraint(rel)]
        else:
            sizes = [_norm(self.fuser.factor(nodes[0]).shape[0])]
        self.webview.evalJS('SIZES = {};'.format(repr(sizes)))

    @QtCore.pyqtSlot(str)
    def _on_graph_element_selected(self, element_id):
        self.on_graph_element_selected(element_id)

    def on_graph_element_selected(self, element_id):
        """Handle graph_element_selected signal, and update the info boxen"""
        if not element_id:
            return self._populate_tables(reset=True)
        selected_is_edge = element_id.startswith('edge ')
        nodes = _get_selected_nodes(element_id, self.fuser.fusion_graph)
        # Update the control listview table
        if selected_is_edge:
            backbones = [(rel, (self.fuser.backbone(rel),))
                         for rel in self.fuser.fusion_graph.get_relations(*nodes)
                         if not is_constraint(rel)]
            self._populate_tables(backbones=backbones)
            self.table_backbones.select_first()
        else:
            self._populate_tables(factors=[(nodes[0], (self.fuser.factor(nodes[0]),))])
            self.table_factors.select_first()

    def _create_layout(self):
        self.mainArea.layout().addWidget(self.webview)
        info = gui.widgetBox(self.controlArea, 'Info')
        gui.label(info, self, '%(n_object_types)d object types')
        gui.label(info, self, '%(n_relations)d relations')
        box = gui.widgetBox(self.controlArea, 'Recipe factors')
        self.table_factors = SimpleTableWidget(box, callback=self.on_selected_factor)
        self.controlArea.layout().addWidget(box)
        box = gui.widgetBox(self.controlArea, 'Backbone factors')
        self.table_backbones = SimpleTableWidget(box, callback=self.on_selected_backbone)
        self.controlArea.layout().addWidget(box)
        box = gui.widgetBox(self.controlArea, 'Completed relations')
        self.table_completions = SimpleTableWidget(box, callback=self.on_selected_completion)
        self.controlArea.layout().addWidget(box)
        self.controlArea.layout().addStretch(1)

    def on_selected_completion(self, item):
        self.table_factors.clearSelection()
        self.table_backbones.clearSelection()
        self.commit(item)

    def on_selected_factor(self, item):
        self.table_completions.clearSelection()
        self.table_backbones.clearSelection()
        self.commit(item)

    def on_selected_backbone(self, item):
        self.table_completions.clearSelection()
        self.table_factors.clearSelection()
        self.commit(item)

    def commit(self, item):
        data = item.data(QtCore.Qt.UserRole)
        self.send(Output.RELATION, Relation.create(*data, graph=self.fuser))

    def _populate_tables(self, factors=None, backbones=None, reset=False):
        self.table_factors.clear()
        self.table_backbones.clear()
        self.send(Output.RELATION, None)
        if factors or reset:
            for otype, matrices in factors or self.fuser.factors_.items():
                M = matrices[0]
                self.table_factors.add(((rel_shape(M.data), (M, otype, None)),
                                        otype.name),
                                       bold=(1,))
        if backbones or reset:
            for relation, matrices in backbones or self.fuser.backbones_.items():
                M = matrices[0]
                self.table_backbones.add([(rel_shape(M.data), (M, None, None))]
                                         + rel_cols(relation),
                                         bold=(1, 3))
        if reset:
            self.table_completions.clear()
            self.table_completions.setRowCount(0)
            for relation, matrices in self.fuser.backbones_.items():
                M = self.fuser.complete(relation)
                self.table_completions.add([(rel_shape(M.data),
                                             (M,
                                              relation.row_type,
                                              relation.col_type))]
                                           + rel_cols(relation),
                                           bold=(1, 3))

    def on_fuser_change(self, fuser):
        self.fuser = fuser
        self._populate_tables(reset=True)
        self.repaint()
        # this ensures gui.label-s get updated
        self.n_object_types = fuser.n_object_types
        self.n_relations = fuser.n_relations

    def repaint(self):
        self.webview.repaint(self.fuser, self)
        self.webview.evalJS(JS_FACTORS)


def main():
    # example from https://github.com/marinkaz/scikit-fusion
    import numpy as np
    R12 = np.random.rand(50, 100)
    R32 = np.random.rand(150, 100)
    R33 = np.random.rand(150, 150)
    t1 = fusion.ObjectType('Users', 10)
    t2 = fusion.ObjectType('Movies', 30)
    t3 = fusion.ObjectType('Actors', 40)
    relations = [fusion.Relation(R12, t1, t2, name='like'),
                 fusion.Relation(R12, t1, t2, name='don\'t like'),
                 fusion.Relation(R33, t3, t3, name='married to'),
                 fusion.Relation(R32, t3, t2, name='play in')]
    G = fusion.FusionGraph()
    for rel in relations:
        G.add_relation(rel)
    fuser = fusion.Dfmf()
    fuser.fuse(G)
    app = QtGui.QApplication([])
    w = OWLatentFactors()
    w.on_fuser_change(FittedFusionGraph(fuser))
    w.show()
    app.exec()


if __name__ == "__main__":
    main()



import sys
from orangecontrib.datafusion import movielens
import numpy as np
from PyQt4 import QtGui
from PyQt4.QtGui import QGridLayout
from Orange.widgets.widget import OWWidget
from Orange.widgets import widget, gui
from orangecontrib.datafusion.models import Relation

from skfusion import fusion


class OWMovieGenres(OWWidget):
    name = "Movie Genres"
    priority = 90000
    icon = "icons/MovieGenres.svg"
    want_main_area = False
    description = "Get a movies-genres or actors-genres matrix"
    inputs = [("Row Type", Relation, "set_data", widget.Default)]
    outputs = [("Genres", Relation, widget.Default)]

    def __init__(self):
        super().__init__()
        self.data = None
        self.matrix = None
        self.genres = None
        self.row_names = None
        self.labels = []
        self.row_type = None
        self.relation_name = ""

        self.layout = QGridLayout()
        self.genrebox = gui.widgetBox(self.controlArea, "Genres")

        self.setSizePolicy(QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,
                           QtGui.QSizePolicy.Fixed))
        self.setMinimumWidth(250)

    def update_genres(self):
        #print("updating", len(self.genres))
        if self.genres is not None:
            for label in self.labels:
                label.hide()
            size = int(np.ceil(np.sqrt(len(self.genres))))
            for i in range(len(self.genres)):
                self.labels.append(gui.widgetLabel(self.genrebox, self.genres[i]))

    def set_data(self, data):
        self.data = data
        if self.data is not None:
            for type_, names in [(data.relation.row_type, data.relation.row_names),
                                 (data.relation.col_type, data.relation.col_names)]:
                if type_.name == "Actors":
                    self.row_type = type_
                    self.row_names = names
                    self.relation_name = "prefer"
                    self.matrix, self.genres = movielens.actor_genre_matrix(actors=self.row_names)
                    break
                elif type_.name == "Movies":
                    self.row_type = type_
                    self.row_names = names
                    self.relation_name = "fit in"
                    self.matrix, self.genres = movielens.movie_concept_matrix(self.row_names, concept="genre")
                    break
            else:
                raise ValueError("Can produce genres only for movies or actors.")

            self.update_genres()
            self.send_output()

    def send_output(self):
        if self.data is not None:
            relation = fusion.Relation(self.matrix, name=self.relation_name,
                                       row_type=self.row_type, row_names=self.row_names,
                                       col_type=fusion.ObjectType("Genres"), col_names=self.genres)
            self.send("Genres", Relation(relation))

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    ow = OWMovieGenres()
    #ow.set_data(movies)
    ow.show()
    app.exec_()



#!/usr/bin/env python

from setuptools import setup, find_packages
import distutils
VERSION = '0.1.1'

ENTRY_POINTS = {
    'orange3.addon': (
        'datafusion = orangecontrib.datafusion',
    ),
    # Entry point used to specify packages containing tutorials accessible
    # from welcome screen. Tutorials are saved Orange Workflows (.ows files).
    'orange.widgets.tutorials': (
        # Syntax: any_text = path.to.package.containing.tutorials
        'fusiontutorials = orangecontrib.datafusion.tutorials',
    ),

    # Entry point used to specify packages containing widgets.
    'orange.widgets': (
        # Syntax: category name = path.to.package.containing.widgets
        # Widget category specification can be seen in
        #    orangecontrib/datafusion/widgets/__init__.py
        'Data Fusion = orangecontrib.datafusion.widgets',
    ),
}

if __name__ == '__main__':
    setup(
        name="Orange3-DataFusion",
        version=VERSION,
        keywords=(
            'orange3 add-on',
        ),
        packages=find_packages(),
        package_data={
            "orangecontrib.datafusion": ["datasets/*.csv"],
            "orangecontrib.datafusion.widgets": ["icons/*.svg", "*.js"],
            "orangecontrib.datafusion.tutorials": ["*.ows"],
        },
        install_requires=[
            'Orange',
            'scikit-fusion',
            'pygraphviz>=1.3rc2',
        ],
        dependency_links = [
            'https://github.com/marinkaz/scikit-fusion/archive/master.zip#egg=scikit-fusion',
        ],
        entry_points=ENTRY_POINTS,
        namespace_packages=['orangecontrib'],
        zip_safe=False,
    )



import sys
import copy
import Orange.data.table
from PyQt4 import QtGui
from PyQt4.QtCore import Qt
from Orange.widgets.widget import OWWidget
from Orange.widgets import widget, gui, settings
from orangecontrib.datafusion.movielens import hide_data, SampleBy
from orangecontrib.datafusion.models import Relation

import numpy as np


class Output:
    IN_SAMPLE_DATA = "In-sample Data"
    OUT_OF_SAMPLE_DATA = "Out-of-sample Data"


class OWSampleMatrix(OWWidget):
    name = "Matrix Sampler"
    priority = 60000
    icon = "icons/MatrixSampler.svg"
    want_main_area = False
    description = "Sample a data matrix"
    inputs = [("Data", Orange.data.table.Table, "set_data", widget.Default)]
    outputs = [(Output.IN_SAMPLE_DATA, Relation),
               (Output.OUT_OF_SAMPLE_DATA, Relation)]

    percent = settings.Setting(90)
    method = settings.Setting(0)
    bools = settings.Setting([])

    def __init__(self):
        super().__init__()
        self.relation = None

        form = QtGui.QGridLayout()
        methodbox = gui.radioButtonsInBox(
            self.controlArea, self, "method", [],
            box=self.tr("Sampling method"), orientation=form)

        rows = gui.appendRadioButton(methodbox, "Rows", addToLayout=False)
        form.addWidget(rows, 0, 0, Qt.AlignLeft)

        cols = gui.appendRadioButton(methodbox, "Columns", addToLayout=False)
        form.addWidget(cols, 0, 1, Qt.AlignLeft)

        rows_and_cols = gui.appendRadioButton(methodbox, "Rows and columns", addToLayout=False)
        form.addWidget(rows_and_cols, 1, 0, Qt.AlignLeft)

        entries = gui.appendRadioButton(methodbox, "Entries", addToLayout=False)
        form.addWidget(entries, 1, 1, Qt.AlignLeft)

        sample_size = gui.widgetBox(self.controlArea, "Proportion of data in the sample")
        percent = gui.hSlider(
            sample_size, self, 'percent', minValue=1, maxValue=100, step=5,
            ticks=10, labelFormat=" %d%%")

        gui.button(self.controlArea, self, "&Apply",
                   callback=self.send_output, default=True)

        self.setSizePolicy(
            QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed,
                              QtGui.QSizePolicy.Fixed))

        self.setMinimumWidth(250)
        self.send_output()

    def set_data(self, data):
        self.relation = data
        self.send_output()

    def send_output(self):
        if self.relation is not None:
            sample_mask, oos_mask = hide_data(self.relation,
                                              self.percent / 100,
                                              SampleBy.all[self.method])
            def _mask_relation(relation, mask):
                if np.ma.is_masked(relation.data):
                    mask = np.logical_or(mask, relation.data.mask)
                data = copy.copy(relation)
                data.data = np.ma.array(data.data, mask=mask)
                return data

            sample_data = _mask_relation(self.relation.relation, sample_mask)
            oos_data = _mask_relation(self.relation.relation, oos_mask)

            self.send(Output.IN_SAMPLE_DATA, Relation(sample_data))
            self.send(Output.OUT_OF_SAMPLE_DATA, Relation(oos_data))


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    ow = OWSampleMatrix()
    # ow.set_data(Orange.data.Table("housing.tab"))
    ow.send_output()
    ow.show()
    app.exec_()
